/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.content.ContentProvider;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

/**
 * Content provider for the DanceDb database.
 * 
 * @author Bronwyn Woods
 *
 */
public class DanceProvider extends ContentProvider {
	private DanceDb mDanceDb;
	private static final String AUTHORITY = "com.dancingwoods.choreo.danceprovider";
	
	private static final int AUTHORS = 1;
	private static final int AUTHOR_ID =2;
	private static final int MOVES = 3;
	private static final int MOVE_ID =4;
	private static final int TAGS = 5;
	private static final int TAG_ID = 6;
	//public static final int AUTHORS_FILTER =7;
	private static final int DANCES_UNIQUE = 8;
	private static final int TAGS_FOR_MOVE_ID = 9;
	private static final int TAG_ID_FOR_MOVE_ID = 10;

	/**
	 * AUTHORS_URI :
	 *   Queries authors table
	 *     authors//# returns author of specified ID
	 *     
	 * MOVES_URI :
	 *   Queries moves table
	 *     moves//# returns move of specified ID
	 *     
	 * TAGS_URI :
	 *   Queries tags table
	 *     tags//# returns tag of specified ID
	 *     tags/move//# returns tags for specified move ID, deletes all tags for a move
	 *     tags//#/move//# returns specific tag if linked with specific move, deletes association between a particular tag and move
	 */
	public static final Uri AUTHORS_URI = Uri.parse("content://" + AUTHORITY + "/" + "authors");
	//public static final Uri AUTHORS_FILTER_URI = Uri.parse("content://" + AUTHORITY +"/"+"authors/filter");
	public static final Uri MOVES_URI = Uri.parse("content://" + AUTHORITY + "/" + "moves");
	public static final Uri TAGS_URI = Uri.parse("content://" + AUTHORITY + "/" + "tags");
	public static final Uri DANCES_UNIQUE_URI = Uri.parse("content://" + AUTHORITY + "/" + "dancesunique");

	
	private static final UriMatcher sURIMatcher = new UriMatcher(UriMatcher.NO_MATCH);
    static
    {
        sURIMatcher.addURI(AUTHORITY, "authors", AUTHORS);
        sURIMatcher.addURI(AUTHORITY, "authors/#", AUTHOR_ID);
        //sURIMatcher.addURI(AUTHORITY, "authors/filter/*", AUTHORS_FILTER);
        sURIMatcher.addURI(AUTHORITY, "moves", MOVES);
        sURIMatcher.addURI(AUTHORITY, "moves/#", MOVE_ID);
        sURIMatcher.addURI(AUTHORITY, "tags", TAGS);
        sURIMatcher.addURI(AUTHORITY, "tags/#", TAG_ID);
        sURIMatcher.addURI(AUTHORITY, "dancesunique", DANCES_UNIQUE);       
        sURIMatcher.addURI(AUTHORITY, "tags/move/#", TAGS_FOR_MOVE_ID);       
        sURIMatcher.addURI(AUTHORITY, "tags/#/move/#", TAG_ID_FOR_MOVE_ID);       
    }
	
    
	@Override
	public int delete(Uri uri, String selection, String[] selectionArgs) {
	    int uriType = sURIMatcher.match(uri);
	    SQLiteDatabase sqlDB = mDanceDb.getWritableDatabase();
	    int rowsAffected = 0;
	    String id = null;
	    
	    switch (uriType) {
	    case AUTHORS:
	        rowsAffected = sqlDB.delete(DanceDb.TABLE_AUTHORS,
	                selection, selectionArgs);
	        break;
	    case AUTHOR_ID:
	        id = uri.getLastPathSegment();
	        if (TextUtils.isEmpty(selection)) {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_AUTHORS,
	                    DanceDb.AUTH_ROWID + "=" + id, null);
	        } else {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_AUTHORS,
	                    selection + " and " + DanceDb.AUTH_ROWID + "=" + id,
	                    selectionArgs);
	        }
	        break;
	    case TAGS:
	        rowsAffected = sqlDB.delete(DanceDb.TABLE_TAGS,
	                selection, selectionArgs);
	        break;
	    case TAG_ID:
	        id = uri.getLastPathSegment();
	        if (TextUtils.isEmpty(selection)) {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_TAGS,
	                    DanceDb.TAG_ROWID + "=" + id, null);
	        } else {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_TAGS,
	                    selection + " and " + DanceDb.TAG_ROWID + "=" + id,
	                    selectionArgs);
	        }
	        break;
	    case MOVES:
	        rowsAffected = sqlDB.delete(DanceDb.TABLE_MOVES,
	                selection, selectionArgs);
	        break;
	    case MOVE_ID:
	        id = uri.getLastPathSegment();
	        if (TextUtils.isEmpty(selection)) {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_MOVES,
	                    DanceDb.MOVE_ROWID + "=" + id, null);
	        } else {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_MOVES,
	                    selection + " and " + DanceDb.MOVE_ROWID + "=" + id,
	                    selectionArgs);
	        }
	        break;
	    case TAGS_FOR_MOVE_ID:
	    	//Deletes all tags for specified move id
	    	id = uri.getLastPathSegment();
	        if (TextUtils.isEmpty(selection)) {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_MOVETAG,
	                    DanceDb.MOVETAG_MOVEID + "=" + id, null);
	        } else {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_MOVETAG,
	                    selection + " and " + DanceDb.MOVETAG_MOVEID + "=" + id,
	                    selectionArgs);
	        }
	        break;
	    case TAG_ID_FOR_MOVE_ID:
	    	//deletes specified tag for specified move id
	    	String moveid = uri.getLastPathSegment();
	    	String tagid = uri.getPathSegments().get(1);
	        if (TextUtils.isEmpty(selection)) {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_MOVETAG,
	                    DanceDb.MOVETAG_MOVEID + "=" + moveid+" and "+DanceDb.MOVETAG_TAGID+"="+tagid, null);
	        } else {
	            rowsAffected = sqlDB.delete(DanceDb.TABLE_MOVETAG,
	                    selection + " and " + DanceDb.MOVETAG_MOVEID + "=" + moveid+" and "+DanceDb.MOVETAG_TAGID+"="+tagid,
	                    selectionArgs);
	        }
	        break;
	    default:
	        throw new IllegalArgumentException("Unknown or Invalid URI " + uri);
	    }
	    
	    getContext().getContentResolver().notifyChange(uri, null);
	    return rowsAffected;	
	}

	@Override
	public String getType(Uri uri) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Uri insert(Uri uri, ContentValues values) {
	    int uriType = sURIMatcher.match(uri);
	    SQLiteDatabase sqlDB = mDanceDb.getWritableDatabase();
	    String id = null;
	    
	    switch (uriType) {
	    case AUTHORS:
	        id = String.valueOf(sqlDB.insertOrThrow(DanceDb.TABLE_AUTHORS, null, values));
	        break;
	    case TAGS:
	        id = String.valueOf(sqlDB.insertOrThrow(DanceDb.TABLE_TAGS, null, values));
	        break;
	    case MOVES:
	        id = String.valueOf(sqlDB.insertOrThrow(DanceDb.TABLE_MOVES, null, values));
	        break;
	    case TAG_ID_FOR_MOVE_ID:
	    	String moveid = uri.getLastPathSegment();
	    	String tagid = uri.getPathSegments().get(1);
	    	id = tagid;
	    	ContentValues cv = new ContentValues();
	    	cv.put(DanceDb.MOVETAG_MOVEID, moveid);
	    	cv.put(DanceDb.MOVETAG_TAGID, tagid);
	    	sqlDB.insertOrThrow(DanceDb.TABLE_MOVETAG, null, cv);
	    	break;
	    default:
	    	Log.d("DanceProvider", String.valueOf(uri));
	    	Log.d("DanceProvider", String.valueOf(uriType));
	        throw new IllegalArgumentException("Unknown or inappropriate URI");
	    }

	    return Uri.withAppendedPath(uri, id);
	}

	
	@Override
	public boolean onCreate() {
		mDanceDb = new DanceDb(getContext());
		return true;
	}


	@Override
	public Cursor query(Uri uri, String[] projection, String selection,
			String[] selectionArgs, String sortOrder) {

		//Log.d("URI being checked", String.valueOf(uri));
		
		//Create the query builder
		SQLiteQueryBuilder queryBuilder = new SQLiteQueryBuilder();

	    // switch on URI type
	    int uriType = sURIMatcher.match(uri);
		switch (uriType) {
	    case AUTHOR_ID:
	    	// Return author table row for particular author ID
		    queryBuilder.setTables(DanceDb.TABLE_AUTHORS);
	        queryBuilder.appendWhere(DanceDb.AUTH_ROWID + "="
	                + uri.getLastPathSegment());
	        break;
	    case AUTHORS:
	    	// Return all author table rows
		    queryBuilder.setTables(DanceDb.TABLE_AUTHORS);
	        break;
	    case DANCES_UNIQUE:
	    	// Return unique values of specified column in dances table
	    	queryBuilder.setTables(DanceDb.TABLE_DANCES);
	    	queryBuilder.setDistinct(true);
	    	break;
	    	
//	    case AUTHORS_FILTER:
//	    	queryBuilder.setTables(DanceDb.TABLE_AUTHORS);
//	    	String[] names = uri.getLastPathSegment().split("\\s+");
//	    	String wherestr = "";
//	    	for(int i =0; i < names.length ; i++){
//	    		if(i==0){
//	    			wherestr = DanceDb.AUTH_FNAME + " LIKE \'%" + names[i] + "%\' OR " + 
//	    					DanceDb.AUTH_LNAME + " LIKE \'%" + names[i]+"%\'";
//	    		}else{
//	    		wherestr = wherestr + " AND " + DanceDb.AUTH_FNAME + " LIKE \'%" + names[i] + "%\' OR " + 
//    					DanceDb.AUTH_LNAME + " LIKE \'%" + names[i]+"%\'";
//	    		}
//	    	}
//	    	queryBuilder.appendWhere(wherestr);
//	    	break;
	    	
	    case TAG_ID:
		    queryBuilder.setTables(DanceDb.TABLE_TAGS);
	        queryBuilder.appendWhere(DanceDb.TAG_ROWID + "="
	                + uri.getLastPathSegment());
	        break;
	    case TAGS:
		    queryBuilder.setTables(DanceDb.TABLE_TAGS);
	        break;
	    case MOVE_ID:
		    queryBuilder.setTables(DanceDb.TABLE_MOVES);
	        queryBuilder.appendWhere(DanceDb.MOVE_ROWID + "="
	                + uri.getLastPathSegment());
	        break;
	    case MOVES:
		    queryBuilder.setTables(DanceDb.TABLE_MOVES);
	        break;
	    case TAGS_FOR_MOVE_ID:
	    	// Return inner join of tags and move tables, for rows with moveid specified
	    	Log.d("DanceProvider", "TAGS_FOR_MOVE_ID");
	    	String moveid = uri.getLastPathSegment();
	    	queryBuilder.setTables(DanceDb.TABLE_TAGS+" INNER JOIN "+DanceDb.TABLE_MOVETAG+
	    			" on "+DanceDb.TABLE_TAGS+"."+DanceDb.TAG_ROWID+"="+DanceDb.MOVETAG_TAGID+" INNER JOIN "+DanceDb.TABLE_MOVES+" on "+
	    			DanceDb.MOVETAG_MOVEID+"="+DanceDb.TABLE_MOVES+"."+DanceDb.MOVE_ROWID);
	    	queryBuilder.appendWhere(DanceDb.TABLE_MOVES+"."+DanceDb.MOVE_ROWID + "=" +moveid);
	    	break;
	    case TAG_ID_FOR_MOVE_ID:
	    	// Return inner join of tags and move tables, for rows with moveid and tagid specified
// NEED to fix the duplicate column name thing here
//	    	Log.d("DanceProvider", "TAG_ID_FOR_MOVE_ID");
//	    	String moveid2 = uri.getLastPathSegment();
//	    	String tagid = uri.getPathSegments().get(1);
//	    	queryBuilder.setTables(DanceDb.TABLE_TAGS+" INNER JOIN "+DanceDb.TABLE_MOVETAG+
//	    			" on "+DanceDb.TAG_ROWID+"="+DanceDb.MOVETAG_TAGID+" INNER JOIN "+DanceDb.TABLE_MOVES+" on "+
//	    			DanceDb.MOVETAG_MOVEID+"="+DanceDb.MOVE_ROWID);
//	    	queryBuilder.appendWhere(DanceDb.MOVE_ROWID + "=" + moveid2);
//	    	queryBuilder.appendWhere(DanceDb.TAG_ROWID+"="+tagid);
//	    	break;

	    default:
	        throw new IllegalArgumentException("Unknown URI");
	    }

		Log.d("CursorProvider query", String.valueOf(uriType));
		Cursor cursor = queryBuilder.query(mDanceDb.getReadableDatabase(),
	            projection, selection, selectionArgs, null, null, sortOrder);
	    cursor.setNotificationUri(getContext().getContentResolver(), uri);
	    return cursor;
	}

	@Override
	public int update(Uri uri, ContentValues values, String selection,
			String[] selectionArgs) {
	    int uriType = sURIMatcher.match(uri);
	    SQLiteDatabase sqlDB = mDanceDb.getWritableDatabase();
	    int rowsAffected = 0;
	    String id = null;
	    
	    switch (uriType) {
	    case AUTHORS:
	        rowsAffected = sqlDB.update(DanceDb.TABLE_AUTHORS, values,
	                selection, selectionArgs);
	        break;
	    case AUTHOR_ID:
	        id = uri.getLastPathSegment();
	        if (TextUtils.isEmpty(selection)) {
	            rowsAffected = sqlDB.update(DanceDb.TABLE_AUTHORS, values,
	                    DanceDb.AUTH_ROWID + "=" + id, null);
	        } else {
	            rowsAffected = sqlDB.update(DanceDb.TABLE_AUTHORS, values,
	                    selection + " and " + DanceDb.AUTH_ROWID + "=" + id,
	                    selectionArgs);
	        }
	        break;
	    case TAGS:
	        rowsAffected = sqlDB.update(DanceDb.TABLE_TAGS, values,
	                selection, selectionArgs);
	        break;
	    case TAG_ID:
	        id = uri.getLastPathSegment();
	        if (TextUtils.isEmpty(selection)) {
	            rowsAffected = sqlDB.update(DanceDb.TABLE_TAGS, values,
	                    DanceDb.TAG_ROWID + "=" + id, null);
	        } else {
	            rowsAffected = sqlDB.update(DanceDb.TABLE_TAGS, values,
	                    selection + " and " + DanceDb.TAG_ROWID + "=" + id,
	                    selectionArgs);
	        }
	        break;
	    case MOVES:
	        rowsAffected = sqlDB.update(DanceDb.TABLE_MOVES, values,
	                selection, selectionArgs);
	        break;
	    case MOVE_ID:
	        id = uri.getLastPathSegment();
	        if (TextUtils.isEmpty(selection)) {
	            rowsAffected = sqlDB.update(DanceDb.TABLE_MOVES, values, 
	                    DanceDb.MOVE_ROWID + "=" + id, null);
	        } else {
	            rowsAffected = sqlDB.update(DanceDb.TABLE_MOVES, values,
	                    selection + " and " + DanceDb.MOVE_ROWID + "=" + id,
	                    selectionArgs);
	        }
	        break;

	    default:
	        throw new IllegalArgumentException("Unknown or Invalid URI " + uri);
	    }
	    
	    getContext().getContentResolver().notifyChange(uri, null);
	    return rowsAffected;	
	}

}
