/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.app.Activity;
import android.app.Fragment;
import android.app.LoaderManager;
import android.content.ContentValues;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnKeyListener;
import android.view.ViewGroup;
import android.widget.EditText;

/**
 * Fragment to hold fields for entering information about a author.
 * 
 * @author Bronwyn Woods
 *
 */
public class FragAuthorEdit extends Fragment implements LoaderManager.LoaderCallbacks<Cursor>{

    private AuthorManipListener mAuthorListener;
    
    private EditText mFnameText;
    private EditText mLnameText;
    private EditText mWebText;
    private EditText mEmailText;
    private Long mRowId;

    
   /**
    * Interface that the container activity must implement.
    * onAuthorUpdate handles saving an author after editing.
    */
    public interface AuthorManipListener {
        public void onAuthorUpdate(Long mRowID, ContentValues values);
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mAuthorListener = (AuthorManipListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement AuthorManipListener method onAuthorUpdate");
        }
    }
    
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
	}


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.frag_author_edit, container, false);
    }
    
 
    @Override    
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        mFnameText = (EditText) getView().findViewById(R.id.auth_editfname);
        mLnameText = (EditText) getView().findViewById(R.id.auth_editlname);
        mEmailText = (EditText) getView().findViewById(R.id.auth_editemail);   
        mWebText = (EditText) getView().findViewById(R.id.auth_editweb);
        mWebText.setOnKeyListener(new OnKeyListener() {
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                // If the event is a key-down event on the "enter" button
                if ((event.getAction() == KeyEvent.ACTION_DOWN) &&
                    (keyCode == KeyEvent.KEYCODE_ENTER)) {
                  // Perform action on key press
                  onSaveAuthor();
                  return true;
                }
                return false;
            }
        });              
     }
    
	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
	    inflater.inflate(R.menu.author_edit_menu, menu);
	}

	
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch(item.getItemId()) {
        case R.id.author_edit_save:
            onSaveAuthor();
            return true;
        }
        return true;
    }

    
    /**
     * Collect data from entry fields and pass to container activity
     * to enter into database.
     */
    public void onSaveAuthor() {
    	//Log.d("FragAuthorEdit","Saving Author!");
		String fname = mFnameText.getText().toString().trim();
		String lname = mLnameText.getText().toString().trim();
		String web = mWebText.getText().toString().trim();
		String email = mEmailText.getText().toString().trim();

		 
		ContentValues values = new ContentValues();
		values.put(DanceDb.AUTH_FNAME, fname);
		values.put(DanceDb.AUTH_LNAME, lname);
		values.put(DanceDb.AUTH_WEB, web);
		values.put(DanceDb.AUTH_EMAIL, email);

		mAuthorListener.onAuthorUpdate(mRowId, values);
    }
 
    
    public void displayAuthor(long rowid){
    	/*
    	 * Init the loader if there isn't already one assigned.
    	 * If there is already a loader, restart it to show the selected author
    	 */
        if(mRowId != null){
        	mRowId = rowid;
        	getLoaderManager().restartLoader(0, null, this);
        }else{
           	mRowId = rowid;
        	getLoaderManager().initLoader(0, null, this);       	
        }
    }
    
    
    /*
     * Methods for loader management 
     */
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
    	CursorLoader cursorLoader=null;
    	if(id==0){
    		// Get author data
	        String[] projection = {DanceDb.AUTH_ROWID, DanceDb.AUTH_FNAME, DanceDb.AUTH_LNAME, DanceDb.AUTH_EMAIL, DanceDb.AUTH_WEB};
	        cursorLoader = new CursorLoader(getActivity(),
	                Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(mRowId)), projection, null, null, null);
    	}
        return cursorLoader;
    }
 
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
    	Integer id = loader.getId();
     	if(id==0){
     		// Data for author
	    	cursor.moveToFirst();
	    	mFnameText.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.AUTH_FNAME)));
	    	mLnameText.setText(String.valueOf(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.AUTH_LNAME))));
	    	mWebText.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.AUTH_WEB)));
	    	mEmailText.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.AUTH_EMAIL)));
    	}
    }
 
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
    	Integer id = loader.getId();
    	if(id==0){
	        mFnameText.setText("");
	        mLnameText.setText("");
	        mEmailText.setText("");
	        mWebText.setText("");
    	}
    }

    
        
}
