/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.app.Activity;
import android.app.ListFragment;
import android.app.LoaderManager;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;

/**
 * Fragment to display list of authors currently in the database.
 * 
 * @author Bronwyn Woods
 *
 */
public class FragAuthorList extends ListFragment implements LoaderManager.LoaderCallbacks<Cursor>{

    private static final int EDIT_ID = Menu.FIRST+1;
    private static final int DELETE_ID = Menu.FIRST+2;
    
    private SimpleCursorAdapter mAuthAdapt;
    private AuthorManipListener mAuthorListener;
    
    /**
     * The container activity must implement this interface.
     * onAuthorSelected - action to take when a list item is selected
     * onAuthorEdit - action to take with context menu edit
     * onAuthorDelete - action to take with context menu delete
     */
    public interface AuthorManipListener {
        public void onAuthorSelected(Uri authorUri);
        public void onAuthorEdit(Uri authorUri);
        public void onAuthorDelete(Uri authorUri);
        public void onAuthorNew();
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mAuthorListener = (AuthorManipListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement AuthorManipListener");
        }
    }
 
    /**
     * Map fields from database onto layout fields using a CursorAdaptor
     */
    @Override    
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        registerForContextMenu(getListView());
       
        getLoaderManager().initLoader(0, null, this);
         
        // Create an array to specify the fields we want to display in the list
        String[] from = new String[]{DanceDb.AUTH_FNAME, DanceDb.AUTH_LNAME};

        // and an array of the fields we want to bind those fields to (in this case just text1)
        int[] to = new int[]{R.id.authrow1, R.id.authrow2};

        // Now create a simple cursor adapter and set it to display
        mAuthAdapt = 
            new SimpleCursorAdapter(getActivity(), R.layout.row_author_list, null, from, to, 0);
        setListAdapter(mAuthAdapt);
    }
    
    @Override
    public void onResume(){
    	super.onResume();
    	getLoaderManager().restartLoader(0,null, this);
    }
    
    public void notifyDataChange(){
    	getLoaderManager().restartLoader(0,null, this);
    }

    
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.frag_author_list, container, false);
    }
    
    /**
     * Context menu appears when a list item is clicked and held
     */
    @Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
	    super.onCreateContextMenu(menu, v, menuInfo);
	    menu.add(0, EDIT_ID, 0, "Edit Author");
	    menu.add(0, DELETE_ID, 0, "Delete Author");
	}
    
    /**
     * Handle context menu selection.
     * Call the appropriate callback method in the container activity
     */
    @Override
	public boolean onContextItemSelected(MenuItem item) {
        AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
        switch(item.getItemId()) {
        case DELETE_ID:
        	mAuthorListener.onAuthorDelete(Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(info.id)));
        	break;
        case EDIT_ID:
        	mAuthorListener.onAuthorEdit(Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(info.id)));
        	break;
        }
        return super.onContextItemSelected(item);
	}
 
    /**
     * Handle a click on a list item.
     * Call appropriate callback method in container activity.
     */
    @Override
	public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        mAuthorListener.onAuthorSelected(Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(id)));
    }


    /** 
     * Method for loader management.
     * Specifies database fields to retrieve.
     */
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
        String[] projection = {DanceDb.AUTH_ROWID, DanceDb.AUTH_FNAME, DanceDb.AUTH_LNAME};
        
        CursorLoader cursorLoader = new CursorLoader(getActivity(),
                DanceProvider.AUTHORS_URI, projection, null, null, DanceDb.AUTH_LNAME);
        return cursorLoader;
    }
 
    /**
     * Specify a cursor for the cursor adaptor.
     */
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
        mAuthAdapt.swapCursor(cursor);
    }
 
    /**
     * Switch the cursor in the cursor adaptor with a null cursor.
     */
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
        mAuthAdapt.swapCursor(null);
    }
    
}
