/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.app.Activity;
import android.app.ListFragment;
import android.app.LoaderManager;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

/**
 * Fragment to hold fields for entering information about a author.
 * 
 * @author Bronwyn Woods
 *
 */
public class FragAuthorView extends ListFragment implements LoaderManager.LoaderCallbacks<Cursor>{

    private AuthorManipListener mAuthorListener;

    private TextView mNameText;
    private TextView mEmailText;
    private TextView mWebText;
    private Long mRowId;
    private SimpleCursorAdapter mDanceAdapt;
    
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
	}

   /**
    * Interface that the container activity must implement.
    * onAuthorUpdate handles saving an author after editing.
    */
    public interface AuthorManipListener {
        public void onAuthorEdit(Uri authorUri);
        public void onAuthorDelete(Uri authorUri);
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mAuthorListener = (AuthorManipListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement AuthorManipListener method onAuthorEdit");
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.frag_author_view, container, false);
    }
    
 
    @Override    
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        mNameText = (TextView) getView().findViewById(R.id.author_view_name);
        mWebText = (TextView) getView().findViewById(R.id.author_view_website);
        mEmailText = (TextView) getView().findViewById(R.id.author_view_email);   
        
        // Create an array to specify the fields we want to display in the list
        String[] from = new String[]{DanceDb.DANCE_NAME};

        // and an array of the fields we want to bind those fields to (in this case just text1)
        int[] to = new int[]{R.id.dance_list_title};

        // Now create a simple cursor adapter and set it to display
        mDanceAdapt = 
            new SimpleCursorAdapter(getActivity(), R.layout.row_dance_list, null, from, to, 0);
        setListAdapter(mDanceAdapt);  
        
     }
    
	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
	    inflater.inflate(R.menu.author_view_menu, menu);
	}

	
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch(item.getItemId()) {
        case R.id.author_view_edit:
        	mAuthorListener.onAuthorEdit(Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(mRowId)));
            return true;
        case R.id.author_view_delete:
        	mAuthorListener.onAuthorDelete(Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(mRowId)));
        }
        return true;
    }

    
    public void displayAuthor(long rowid){
    	/*
    	 * Init the loader if there isn't already one assigned.
    	 * If there is already a loader, restart it to show the selected author
    	 */
        if(mRowId != null){
        	mRowId = rowid;
        	getLoaderManager().restartLoader(0, null, this);
        	getLoaderManager().restartLoader(2, null, this);
        }else{
           	mRowId = rowid;
            // Loader to get author data
            getLoaderManager().initLoader(0, null, this);
            // Loader to get dance list
            getLoaderManager().initLoader(1, null, this);
        }
    }
    
    
    /*
     * Methods for loader management 
     */
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
    	CursorLoader cursorLoader=null;
    	if(id==0){
    		// Get author data
	        String[] projection = {DanceDb.AUTH_ROWID, DanceDb.AUTH_FNAME, DanceDb.AUTH_LNAME, DanceDb.AUTH_EMAIL, DanceDb.AUTH_WEB};
	        cursorLoader = new CursorLoader(getActivity(),
	                Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(mRowId)), projection, null, null, null);
    	}else if(id==1){
    		// Get list of dances
    	}
        return cursorLoader;
    }
 
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
    	Integer id = loader.getId();
     	if(id==0){
     		// Data for author
	    	cursor.moveToFirst();
	    	mNameText.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.AUTH_FNAME))+
	                " "+cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.AUTH_LNAME)));
	    	mWebText.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.AUTH_WEB)));
	    	mEmailText.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.AUTH_EMAIL)));
    	}else if(id==1){
    	}
    }
 
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
    	Integer id = loader.getId();
    	if(id==0){
    		// Loader for author data
	        mNameText.setText("");
	        mWebText.setText("");
	        mEmailText.setText("");
    	}else if(id==1){
    		// Loader for dance list
    	}
    }

    
        
}
