/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import java.util.ArrayList;

import android.app.Activity;
import android.app.ListFragment;
import android.app.LoaderManager;
import android.content.ContentValues;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.util.SparseBooleanArray;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

/**
 * Fragment to hold fields for entering information about a move.
 * 
 * @author Bronwyn Woods
 *
 */
public class FragMoveEdit extends ListFragment implements LoaderManager.LoaderCallbacks<Cursor>{

    private MoveManipListener mMoveListener;
    
    private EditText mDescText;
    private EditText mCountText;
    private EditText mSh1Text;
    private EditText mSh2Text;
    private Long mRowId;
    private ListView mTagList;
    private ArrayList<TagListObject> mTags;
    //private ArrayAdapter<TagListObject> mTagAdapt;
    private SimpleCursorAdapter mTagAdapt;
    
   /**
    * Interface that the container activity must implement.
    * onMoveUpdate handles saving an author after editing.
    */
    public interface MoveManipListener {
        public void onMoveUpdate(Long mRowID, ContentValues values, ArrayList<Long> tagids);
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mMoveListener = (MoveManipListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement MoveManipListener method onMoveUpdate");
        }
    }
    
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
	}


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.frag_move_edit, container, false);
    }

    @Override
    public void onResume(){
    	super.onResume();
        mCountText.requestFocus();
    }
 
    @Override    
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        mDescText = (EditText) getView().findViewById(R.id.move_edit_desc);
        mCountText = (EditText) getView().findViewById(R.id.move_edit_count);
        mSh1Text = (EditText) getView().findViewById(R.id.move_edit_sh1);
        mSh2Text = (EditText) getView().findViewById(R.id.move_edit_sh2);   
        mTagList = (ListView) getListView();
        mTags = new ArrayList<TagListObject>();
        
        //Loader for tag data
        getLoaderManager().initLoader(1, null, this);
        
        // Create an array to specify the fields we want to display in the list
        String[] from = new String[]{DanceDb.TAG_DESC, DanceDb.TAG_CAT};

        // and an array of the fields we want to bind those fields to (in this case just text1)
        int[] to = new int[]{R.id.tag_list_desc, R.id.tagcatlabs};

        // Now create a simple cursor adapter and set it to display
        mTagAdapt = 
            new SimpleCursorAdapter(getActivity(), R.layout.row_tag_check, null, from, to, 0);
        mTagAdapt.setViewBinder(new SimpleCursorAdapter.ViewBinder() {
        	  @Override
        	  public boolean setViewValue(View view, Cursor cursor, int columnIndex) {
        		  	int catind = cursor.getColumnIndexOrThrow(DanceDb.TAG_CAT);
					if(catind == columnIndex && view.getId()==R.id.tagcatlabs){
	        		  	
						TextView tview = (TextView)view;
						tview.setText(cursor.getString(columnIndex));
	        		  	String thisCat = cursor.getString(columnIndex);
						String prevCat = null;
						
						// previous row, for comparison
						if (cursor.getPosition() > 0 && cursor.moveToPrevious()) {
						    prevCat = cursor.getString(columnIndex);
						    cursor.moveToNext();
						}
						
						// enable section heading if it's the first one, or 
						// different from the previous one
						if (prevCat == null || !prevCat.equals(thisCat)) {
						    tview.setVisibility(View.VISIBLE);
						} else {
						    tview.setVisibility(View.GONE);
						
						}
						
						return true;
					}
					return false;
        	  }
       	});
        setListAdapter(mTagAdapt);
        // Now create a simple adapter and set it to display
//        mTagAdapt = new ArrayAdapter<TagListObject>(getActivity().getApplicationContext(), R.layout.row_tag_check, R.id.tag_list_desc, mTags);
//        setListAdapter(mTagAdapt);
        mTagList.setDivider(null);
        mTagList.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE);
        //mChTags = new ArrayList<Long>();
        mTagList.setItemsCanFocus(false);
     }
    
	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
	    inflater.inflate(R.menu.move_edit_menu, menu);
	}

	
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch(item.getItemId()) {
        case R.id.move_edit_save:
            onSaveMove();
            return true;
        }
        return true;
    }

    
    /**
     * Collect data from entry fields and pass to container activity
     * to enter into database.
     */
    public void onSaveMove() {
    	//Log.d("FragMoveEdit","Saving Move!");
		String desc = mDescText.getText().toString().trim();
		Integer count = null;
		if(mCountText.getText().toString().length()>0){
			count = Integer.parseInt(mCountText.getText().toString());
		}
		String short1 = mSh1Text.getText().toString().trim();
		String short2 = mSh2Text.getText().toString().trim();
		
		
		SparseBooleanArray cids = mTagList.getCheckedItemPositions();
		//Log.d("FragMoveEdit", "cids size: "+String.valueOf(cids.size()));
		ArrayList<Long> tagids = new ArrayList<Long>();
		for(int i=0; i<cids.size(); i++){
			if(cids.valueAt(i)){
				final int pos = cids.keyAt(i);
				tagids.add(mTags.get(pos).getID());
				//Log.d("Adding tag", String.valueOf(mTags.get(pos).getID()));
			}
		}
		 
		ContentValues values = new ContentValues();
		values.put(DanceDb.MOVE_DESC, desc);
		values.put(DanceDb.MOVE_COUNT, count);
		values.put(DanceDb.MOVE_SHORT1, short1);
		values.put(DanceDb.MOVE_SHORT2, short2);
		mMoveListener.onMoveUpdate(mRowId, values, tagids);
    }
 
    
    public void displayMove(long rowid){
    	/*
    	 * Init the loader if there isn't already one assigned.
    	 * If there is already a loader, restart it to show the selected move
    	 */
        if(mRowId != null){
        	mRowId = rowid;
        	getLoaderManager().restartLoader(0, null, this);
        	getLoaderManager().restartLoader(2, null, this);
        }else{
           	mRowId = rowid;
        	getLoaderManager().initLoader(0, null, this);       	
        	getLoaderManager().initLoader(2, null, this);       	
        }
    }
    
    
    /*
     * Methods for loader management 
     */
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
    	CursorLoader cursorLoader=null;
    	if(id==0){
    		// Get move data
	        String[] projection = {DanceDb.MOVE_ROWID, DanceDb.MOVE_COUNT, DanceDb.MOVE_DESC, DanceDb.MOVE_SHORT1, DanceDb.MOVE_SHORT2};
	        cursorLoader = new CursorLoader(getActivity(),
	                Uri.withAppendedPath(DanceProvider.MOVES_URI, String.valueOf(mRowId)), projection, null, null, null);
    	}else if(id==1){
    		// Get list of tags
	        String[] projection = {DanceDb.TAG_ROWID, DanceDb.TAG_DESC, DanceDb.TAG_CAT};
	        cursorLoader = new CursorLoader(getActivity(),
	                DanceProvider.TAGS_URI, projection, null, null, DanceDb.TAG_CAT+","+DanceDb.TAG_DESC);
    	}else if(id==2){
    		// Get tags for this move
    		Log.d("FragMoveEdit","Getting tags in onCreateLoader");
    		String[] projection = {DanceDb.MOVETAG_MOVEID, DanceDb.MOVETAG_TAGID};
    		cursorLoader = new CursorLoader(getActivity(),
    				Uri.parse(DanceProvider.TAGS_URI+"/move/"+String.valueOf(mRowId)), projection, null, null, null);
    	}
        return cursorLoader;
    }
 
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
    	Integer id = loader.getId();
     	if(id==0){
     		// Data for move
	    	cursor.moveToFirst();
	    	mDescText.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.MOVE_DESC)));
	    	mCountText.setText(String.valueOf(cursor.getInt(
	                cursor.getColumnIndexOrThrow(DanceDb.MOVE_COUNT))));
	    	mSh1Text.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.MOVE_SHORT1)));
	    	mSh2Text.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.MOVE_SHORT2)));
    	}else if(id==1){
    		// Getting list of tags
    		cursor.moveToFirst();
    		while(!cursor.isAfterLast()){
    			mTags.add(new TagListObject(cursor.getLong(cursor.getColumnIndexOrThrow(DanceDb.TAG_ROWID)), 
    					cursor.getString(cursor.getColumnIndexOrThrow(DanceDb.TAG_DESC)), false));
    			cursor.moveToNext();
    		}
    		mTagAdapt.swapCursor(cursor);
    	}else if(id==2){
    		// Tags for this move
    		cursor.moveToFirst();
    		while(!cursor.isAfterLast()){
    			Long tagid = cursor.getLong(cursor.getColumnIndexOrThrow(DanceDb.MOVETAG_TAGID));
    			//Log.d("FragMoveEdit", String.valueOf(tagid));
    			for(int i=0; i<mTags.size(); i++){
    				if(mTags.get(i).getID()==tagid){
    					//mTags.get(i).setSelected(true);
    					mTagList.setItemChecked(i, true);
    					break;
    				}
    			}
    			cursor.moveToNext(); 
    		}
    	}
    }
 
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
    	Integer id = loader.getId();
    	if(id==0){
	        mDescText.setText("");
	        mCountText.setText("");
	        mSh1Text.setText("");
	        mSh2Text.setText("");
    	}else if(id==1){
    		// Getting list of tags
    		mTags.clear();
    		mTagAdapt.swapCursor(null);
    	}else if(id==2){
			for(int i=0; i<mTags.size(); i++){
				mTagList.setItemChecked(i, false);
			}
    	}
    }

    
        
}
