/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.app.Activity;
import android.app.ListFragment;
import android.app.LoaderManager;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;

/**
 * Fragment to display the list of moves currently in the database.
 * 
 * @author Bronwyn Woods
 *
 */
public class FragMoveList extends ListFragment implements LoaderManager.LoaderCallbacks<Cursor>{

    private static final int EDIT_ID = Menu.FIRST+1;
    private static final int DELETE_ID = Menu.FIRST+2;
    
    private SimpleCursorAdapter mMoveAdapt;
    private MoveManipListener mMoveListener;
    
    /**
     * Container activity must implement this interface.
     * These methods hadle interactions with the list items.
     */
    public interface MoveManipListener {
        public void onMoveSelected(Uri moveUri);
        public void onMoveEdit(Uri moveUri);
        public void onMoveDelete(Uri moveUri);
        public void onMoveNew();
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mMoveListener = (MoveManipListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement MoveManipListener");
        }
    }
 
    /**
     * Create a cursor adaptor for the list to handle getting data from the database
     */
    @Override    
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        registerForContextMenu(getListView());
       
        getLoaderManager().initLoader(0, null, this);
         
        // Create an array to specify the fields we want to display in the list
        String[] from = new String[]{DanceDb.MOVE_COUNT, DanceDb.MOVE_DESC, DanceDb.MOVE_SHORT1};

        // and an array of the fields we want to bind those fields to (in this case just text1)
        int[] to = new int[]{R.id.movecount, R.id.movedesc, R.id.moveshort};

        // Now create a simple cursor adapter and set it to display
        mMoveAdapt = 
            new SimpleCursorAdapter(getActivity(), R.layout.row_move_list, null, from, to, 0);
        setListAdapter(mMoveAdapt);
                
    }
    
    public void notifyDataChange(){
    	getLoaderManager().restartLoader(0,null, this);
    }
    
    @Override
    public void onResume(){
    	super.onResume();
		Log.d("testlist1", "this is a test");
    	getLoaderManager().restartLoader(0,null, this);
    }
    
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.frag_move_list, container, false);
    }
    

    @Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
	    super.onCreateContextMenu(menu, v, menuInfo);
	    menu.add(0, EDIT_ID, 0, "Edit Move");
	    menu.add(0, DELETE_ID, 0, "Delete Move");
	}
    
    @Override
	public boolean onContextItemSelected(MenuItem item) {
        AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
        switch(item.getItemId()) {
        case DELETE_ID:
        	mMoveListener.onMoveDelete(Uri.withAppendedPath(DanceProvider.MOVES_URI, String.valueOf(info.id)));
        	break;
        case EDIT_ID:
        	mMoveListener.onMoveEdit(Uri.withAppendedPath(DanceProvider.MOVES_URI, String.valueOf(info.id)));
        	break;
        }
        return super.onContextItemSelected(item);
	}
 
    
    @Override
	public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        mMoveListener.onMoveSelected(Uri.withAppendedPath(DanceProvider.MOVES_URI, String.valueOf(id)));
    }


    /* Methods for loader management*/
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
        String[] projection = {DanceDb.MOVE_ROWID, DanceDb.MOVE_DESC, DanceDb.MOVE_COUNT, DanceDb.MOVE_SHORT1, DanceDb.MOVE_SHORT2};
        
        CursorLoader cursorLoader = new CursorLoader(getActivity(),
                DanceProvider.MOVES_URI, projection, null, null, ""+DanceDb.MOVE_COUNT+", "+DanceDb.MOVE_DESC);
        return cursorLoader;
    }
 
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
        mMoveAdapt.swapCursor(cursor);
    }
 
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
        mMoveAdapt.swapCursor(null);
    }
    
}
