/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import java.util.ArrayList;

import android.app.Activity;
import android.app.Fragment;
import android.app.LoaderManager;
import android.content.ContentValues;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnKeyListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.EditText;

/**
 * Fragment to hold fields for entering or editing information about a author.
 * 
 * @author Bronwyn Woods
 *
 */
public class FragTagEdit extends Fragment implements LoaderManager.LoaderCallbacks<Cursor>{

    private TagManipListener mTagListener;
    
    private EditText mDesc;
    private AutoCompleteTextView mCat;
    private Long mRowId;
    private ArrayList<String> mCatList;
    private ArrayAdapter<String> mCatAdapt;


    
   /**
    * Interface that the container activity must implement.
    * onAuthorUpdate handles saving an author after editing.
    */
    public interface TagManipListener {
        public void onTagUpdate(Long mRowID, ContentValues values);
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mTagListener = (TagManipListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement TagManipListener method onTagUpdate");
        }
    }
    
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
	}


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.frag_tag_edit, container, false);
    }
    
 
    @Override    
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        mCat = (AutoCompleteTextView) getView().findViewById(R.id.tag_editcat);
        mCatList = new ArrayList<String>();
        
        //Loader for category list
        getLoaderManager().initLoader(1, null, this);
        
        // Now create a simple cursor adapter and set it to display
        mCatAdapt = new ArrayAdapter<String>(getActivity().getApplicationContext(), R.layout.dropdown_item, mCatList);
        mCat.setAdapter(mCatAdapt);
        
        mDesc = (EditText) getView().findViewById(R.id.tag_editdesc);
        mDesc.setOnKeyListener(new OnKeyListener() {
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                // If the event is a key-down event on the "enter" button
                if ((event.getAction() == KeyEvent.ACTION_DOWN) &&
                    (keyCode == KeyEvent.KEYCODE_ENTER)) {
                  // Perform action on key press
                  onSaveTag();
                  return true;
                }
                return false;
            }
        });              
     }
    
	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
	    inflater.inflate(R.menu.tag_edit_menu, menu);
	}

	
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch(item.getItemId()) {
        case R.id.tag_edit_save:
            onSaveTag();
            return true;
        }
        return true;
    }

    
    /**
     * Collect data from entry fields and pass to container activity
     * to enter into database.
     */
    public void onSaveTag() {
    	//Log.d("FragAuthorEdit","Saving Author!");
		String cat = mCat.getText().toString().trim();
		String desc = mDesc.getText().toString().trim();
		 
		ContentValues values = new ContentValues();
		values.put(DanceDb.TAG_CAT, cat);
		values.put(DanceDb.TAG_DESC, desc);

		mTagListener.onTagUpdate(mRowId, values);
    }
 
    
    public void displayTag(long rowid){
    	/*
    	 * Init the loader if there isn't already one assigned.
    	 * If there is already a loader, restart it to show the selected author
    	 */
        if(mRowId != null){
        	mRowId = rowid;
        	getLoaderManager().restartLoader(0, null, this);
        }else{
           	mRowId = rowid;
        	getLoaderManager().initLoader(0, null, this);       	
        }
    }
    
    
    /*
     * Methods for loader management 
     */
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
    	CursorLoader cursorLoader=null;
    	if(id==0){
    		// Get author data
	        String[] projection = {DanceDb.TAG_ROWID, DanceDb.TAG_CAT, DanceDb.TAG_DESC};
	        cursorLoader = new CursorLoader(getActivity(),
	                Uri.withAppendedPath(DanceProvider.TAGS_URI, String.valueOf(mRowId)), projection, null, null, null);
    	}else if(id==1){
    		String[] projection = {DanceDb.TAG_ROWID, DanceDb.TAG_CAT};
    		cursorLoader = new CursorLoader(getActivity(),
	                DanceProvider.TAGS_URI, projection, null, null, null);    				
    	}
        return cursorLoader;
    }
 
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
    	Integer id = loader.getId();
     	if(id==0){
     		// Data for tag
	    	cursor.moveToFirst();
	    	mCat.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.TAG_CAT)));
	    	mDesc.setText(String.valueOf(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.TAG_DESC))));
    	}else if(id==1){
    		// Getting list of tags
    		cursor.moveToFirst();
    		while(!cursor.isAfterLast()){
    			String cat = cursor.getString(cursor.getColumnIndexOrThrow(DanceDb.TAG_CAT));
    			if(!mCatList.contains(cat)){
    				mCatList.add(cat);
    			}	
    			cursor.moveToNext();
    		}
            mCatAdapt.notifyDataSetChanged();
    	}
    }
 
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
    	Integer id = loader.getId();
    	if(id==0){
	        mDesc.setText("");
	        mCat.setText("");
    	}else if(id==1){
    		// Getting list of tags
    		mCatList.clear();
    		mCatAdapt.notifyDataSetChanged();
    	}
    }

    
        
}
